//////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2007-2010, Image Engine Design Inc. All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are
//  met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//
//     * Neither the name of Image Engine Design nor the names of any
//       other contributors to this software may be used to endorse or
//       promote products derived from this software without specific prior
//       written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
//  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
//  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
//  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef IE_CORE_MATRIXMOTIONTRANSFORM_H
#define IE_CORE_MATRIXMOTIONTRANSFORM_H

#include "IECore/Export.h"
#include "IECore/Transform.h"

namespace IECore
{

class IECORE_API MatrixMotionTransform : public Transform
{
	public:

		/// A type to map between points in time and
		/// corresponding matrices.
		typedef std::map<float, Imath::M44f> SnapshotMap;

		IE_CORE_DECLAREOBJECT( MatrixMotionTransform, Transform );

		MatrixMotionTransform();
		~MatrixMotionTransform() override;

		void render( Renderer *renderer ) const override;
		Imath::M44f transform( float time = 0 ) const override;

		//! @name Snapshots
		////////////////////////////////////////////
		//@{
		const SnapshotMap &snapshots() const;
		SnapshotMap &snapshots();
		//@}

	private:

		static const unsigned int m_ioVersion;

		SnapshotMap m_snapshots;

};

IE_CORE_DECLAREPTR( MatrixMotionTransform );

}

#endif // IE_CORE_MATRIXMOTIONTRANSFORM_H
