##########################################################################
#
#  Copyright (c) 2009-2012, Image Engine Design Inc. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are
#  met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
#     * Neither the name of Image Engine Design nor the names of any
#       other contributors to this software may be used to endorse or
#       promote products derived from this software without specific prior
#       written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
#  IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##########################################################################

import IECore

def __parseDateTime( args, parameter ) :

	import datetime

	if not len( args ) :
		raise SyntaxError( "Expected date/time." )


	dateStr = args[0]
	if not type( dateStr ) is str :
		raise SyntaxError( "Expected date/time." )

	for format in [ "%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H:%M", "%Y-%m-%d" ] :

		try :
			date = datetime.datetime.strptime( dateStr, format )
			d = IECore.DateTimeData( date )
			parameter.setValidatedValue( d )
			del args[0]
			return
		except :
			pass

	# If no day specified, use today
	for format in [ "%H:%M:%S", "%H:%M" ] :

		try :
			date = datetime.datetime.strptime( dateStr, format )
			today = datetime.datetime.now()
			date = date.replace( day = today.day, month = today.month, year = today.year )
			d = IECore.DateTimeData( date )
			parameter.setValidatedValue( d )
			del args[0]
			return
		except :
			pass

	raise SyntaxError( "Not a valid date/time: '%s'" % ( dateStr ) )


def __serialiseDateTime( parameter, value ) :

	# strptime does not support fractional seconds
	dt = value.value.replace( microsecond = 0 )

	return [ str( dt ) ]

IECore.ParameterParser.registerType( IECore.DateTimeParameter.staticTypeId(), __parseDateTime, __serialiseDateTime )

__all__ = []
